;---------------------------------------------------------------------
; ASSEMBLY LANGUAGE AES DEMO PROGRAM
;
; DEVELOPED WITH DEVPAC2 ST.
; COPYRIGHT 1992 BY CLAYTON WALNUM
;
; NOTES: In order to keep this program listing as understandable as
;        possible, no effort has been made to program "efficiently."
;        For example, there are much better ways of calling AES
;        functions than loading the control array "by hand" for each
;        call.  However, programming tricks like this may obscure
;        what is meant to be crystal clear, so they have been avoided.
;---------------------------------------------------------------------

	include AESDEMO.H

PTERM0          equ     0
GETREZ		equ	4
DGETPATH	equ	71
MSHRINK         equ     74

APPL_INIT       equ     10
APPL_EXIT	equ	19
EVNT_MULTI	equ	25
MENU_BAR	equ	30
MENU_IENABLE	equ	32
MENU_TNORMAL	equ	33
OBJC_DRAW	equ	42
FORM_DO		equ	50
FORM_DIAL	equ	51
FORM_ALERT	equ	52
FORM_CENTER	equ	54
GRAF_HANDLE     equ     77
GRAF_MOUSE	equ	78
GRAF_SHRINKBOX	equ	74
GRAF_GROWBOX	equ	73
FSEL_INPUT	equ	90
WIND_CREATE	equ	100
WIND_OPEN	equ	101
WIND_CLOSE	equ	102
WIND_DELETE	equ	103
WIND_GET	equ	104
WIND_SET	equ	105
WIND_UPDATE	equ	107
RSRC_LOAD	equ	110
RSRC_FREE	equ	111
RSRC_GADDR	equ	112

VSF_COLOR	equ	25
VSF_INTERIOR	equ	23
VSF_STYLE	equ	24
VSWR_MODE	equ	32
V_OPNVWK        equ     100
V_CLSVWK        equ     101
VR_RECFL	equ	114

AES_OPCODE      equ     200
VDI_OPCODE      equ     115

BEG_UPDATE	equ	1
END_UPDATE	equ	0
FMD_FINISH	equ	3
FMD_GROW	equ	1
FMD_SHRINK	equ	2
FMD_START	equ	0
MN_SELECTED	equ	0
WF_CURRXYWH	equ	5
WF_HSLIDE	equ	8
WF_HSLSIZE	equ	15
WF_INFO		equ	3
WF_NAME		equ	2
WF_NEXTXYWH	equ	12
WF_PREVXYWH	equ	6
WF_VSLIDE	equ	9
WF_VSLSIZE	equ	16
WF_WORKXYWH	equ	4
WM_CLOSED	equ	22
WM_FULLED	equ	23
WM_MOVED	equ	28
WM_REDRAW	equ	20
WM_SIZED	equ	27

M_OFF		equ	256
M_ON		equ	257

ARROW		equ	0
LOWREZ		equ	0
R_TREE		equ	0
FALSE		equ	0
TRUE		equ	1
REPLACE		equ	1
SOLID		equ	1
WHITE		equ	0
YES		equ	1
MIN_WIDTH	equ	64
MIN_HEIGHT	equ	64

;--------------------------------------------------------------------
; This macro gets the address of an object's string. It requires two 
; parameters: the address of the resource tree and the object's
; number. The string address is returned in a5.
;-------------------------------------------------------------------- 
tedinfo_str     macro
        ifc     '','\2'
        fail    Missing parameter!
        mexit
        endc
        move.l  \1,a5                   ; Get address of dialog.
        add.l   #(\2*24)+12,a5          ; Calc. addr of ob_spec.
        move.l  (a5),a5                 ; Get address of tedinfo.
        move.l	(a5),a5			; Get address of string.
        endm

;--------------------------------------------------------------------
; MAIN PROGRAM
;--------------------------------------------------------------------
        text

; Calculate the size of the program area.

        move.l  a7,a5                   ; Save addr of TPA.
        lea     stack,sp                ; Load addr of our stack.
        
        move.l  4(a5),a5                ; Get addr of TPA.              
        move.l  12(a5),d0               ; Get len of text segment.
        add.l   20(a5),d0               ; Add len of data segment.
        add.l   28(a5),d0               ; Add len of BSS segment.
        add.l   #$100,d0                ; Add len of TPA. 

; Release unused memory back to the system.
        
        move.l  d0,-(sp)                ; Push size of program on stack.
        move.l  a5,-(sp)                ; Push program addr on stack.
        clr     -(sp)                   ; Clear dummy word on stack.    
        move    #MSHRINK,-(sp)          ; Push Mshrink opcode.
        trap    #1                      ; Call GEMDOS.
        add.l   #12,sp                  ; Reset stack pointer.

; Clear some fields of the global array.

        clr.l   ap_ptree        
        clr.l   ap_1resv
        clr.l   ap_2resv
        clr.l   ap_3resv
        clr.l   ap_4resv

; Call appl_init to initialize application.

        move    #APPL_INIT,control
        move    #0,control+2
        move    #1,control+4
        move    #0,control+6
        move    #0,control+8
        bsr     aes
        cmpi    #$FFFF,ap_id            ; Error?
        beq     end                     ; Yep.  Outta here.

; Get handle to screen device.

        move    #GRAF_HANDLE,control
        move    #0,control+2
        move    #5,control+4
        move    #0,control+6
        move    #0,control+8
        jsr     aes
        move    int_out,gr_handle       ; Save device handle.

; Open virtual workstation.

        move	#V_OPNVWK,control
        clr     control+2
        move    #11,control+6
        move    gr_handle,control+12    
        movea.l #int_in,a5              ; Get address of int_in.
        move    #9,d5                   ; Init loop counter.
loop:
        move    #1,(a5)+                ; Init int_in array.
        dbra    d5,loop
        move    #2,int_in+20
        jsr     vdi
        move    control+12,vws_handle   ; Save virtual device handle.

; Check screen resolution

	move	#GETREZ,-(sp)
	trap	#14
	addq.l	#2,sp
	cmpi	#LOWREZ,d0		; Compare low rez with current rez.
	bne	rez_okay		; If not low rez, continue.

; If rez no good, show alert and close application

	move	#FORM_ALERT,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#1,int_in
	move.l	#alert1,addr_in
	jsr	aes
	bra	close_down
	
rez_okay:

; Reset mouse pointer to arrow..

	move	#GRAF_MOUSE,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#ARROW,int_in
	move.l	#0,addr_in
	jsr	aes

; Load the resource file

	move	#RSRC_LOAD,control
	move	#0,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move.l	#rsrcfile,addr_in
	jsr	aes
	tst	int_out			; Check for an error.
	bne	rsrc_okay

; If got error, show alert and close application.
	
	move	#FORM_ALERT,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#1,int_in
	move.l	#alert2,addr_in
	jsr	aes
	bra	close_down

rsrc_okay:

; Get the addresses of the menu bar and dialog box.

	move	#RSRC_GADDR,control
	move	#2,control+2
	move	#1,control+4
	move	#0,control+6
	move	#1,control+8
	move	#R_TREE,int_in
	move	#MENU,int_in+2
	jsr	aes
	move.l	addr_out,menu_addr	; Save address of menu.
	
	move	#RSRC_GADDR,control
	move	#2,control+2
	move	#1,control+4
	move	#0,control+6
	move	#1,control+8
	move	#R_TREE,int_in
	move	#DIALOG,int_in+2
	jsr	aes
	move.l	addr_out,dialog_addr	; Save address of dialog.

; Show the menu bar.

	move	#MENU_BAR,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#TRUE,int_in
	move.l	menu_addr,addr_in
	jsr	aes

; Disable the "Open" menu item.

	move	#MENU_IENABLE,control
	move	#2,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#OPEN,int_in
	move	#FALSE,int_in+2
	move.l	menu_addr,addr_in
	jsr	aes

; Get the size of the desktop work area.

	move	#WIND_GET,control
	move	#2,control+2
	move	#5,control+4
	move	#0,control+6
	move	#0,control+8
	move	#0,int_in		; desktop window handle.
	move	#WF_WORKXYWH,int_in+2
	jsr	aes
	move	int_out+2,fullx		; Save size of full work area.
	move	int_out+4,fully
	move	int_out+6,fullw
	move	int_out+8,fullh

; Create new window.

	move	#WIND_CREATE,control
	move	#5,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	#4095,int_in		; Window parts flags.
	move	fullx,int_in+2		; Use size of work area for window.
	move	fully,int_in+4
	move	fullw,int_in+6
	move	fullh,int_in+8
	jsr	aes
	move	int_out,w_h1		; Save window handle.

; Set window's name text.

	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_NAME,int_in+2
	move.l	#wname,int_in+4
	jsr	aes

; Set window's information line.

	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_INFO,int_in+2
	move.l	#winfo,int_in+4
	jsr	aes

; Open the new window.

	move	#WIND_OPEN,control
	move	#5,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	fullx,int_in+2
	move	fully,int_in+4
	move	fullw,int_in+6
	move	fullh,int_in+8
	jsr	aes

; Set size of horizontal slider.

	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_HSLSIZE,int_in+2
	move	#500,int_in+4		; Size = half full size.
	jsr	aes
	
; Set position of horizontal slider.

	move	#WF_HSLIDE,int_in+2
	move	#1,int_in+4		; All the way to left.
	jsr	aes
	
; Set size of vertical slider.

	move	#WF_VSLSIZE,int_in+2
	move	#500,int_in+4
	jsr	aes
	
; Set position of vertical slider.

	move	#WF_VSLIDE,int_in+2
	move	#1,int_in+4
	jsr	aes

	move	#FALSE,closed		; Init window-closed flag.

; Get event messages.

get_event:
	move	#EVNT_MULTI,control
	move	#16,control+2
	move	#7,control+4
	move	#1,control+6
	move	#0,control+8
	move	#16,int_in		; Wait for message events.
	move	#0,int_in+2 		; # of clicks to wait for.
	move	#0,int_in+4	 	; Mouse buttons to wait for.
	move	#0,int_in+6		; Button state to wait for.
	move	#0,int_in+8 		; Enter/leave flag rec. 1.
	move	#0,int_in+10 		; X coord. of rec. 1.
	move	#0,int_in+12 		; Y coord. of rec. 1.
	move	#0,int_in+14 		; Width of rec. 1.
	move	#0,int_in+16 		; Height of rec. 1.
	move	#0,int_in+18 		; Enter/leave flag rec. 2
	move	#0,int_in+20 		; X coord. of rec. 2.
	move	#0,int_in+22 		; Y coord. of rec. 2.
	move	#0,int_in+24 		; Width of rec. 2.
	move	#0,int_in+26 		; Height of rec. 2.
	move	#0,int_in+28 		; Low word of timer value.
	move	#0,int_in+30 		; High word of timer value.
	move.l	#msgbuf,addr_in 	; Address of message buffer.
	jsr	aes

	cmpi	#16,int_out		; Got Message event?
	bne	get_event		; Nope.

; Translate message.

	cmpi	#MN_SELECTED,msgbuf
	beq	do_menu
	cmpi	#WM_REDRAW,msgbuf
	beq	do_redraw
	cmpi	#WM_CLOSED,msgbuf
	bne	w1
	jsr	do_close
	bra	get_event
w1:	
	cmpi	#WM_FULLED,msgbuf
	beq	do_full
	cmpi	#WM_MOVED,msgbuf
	beq	do_move
	cmpi	#WM_SIZED,msgbuf
	beq	do_move
	
do_menu:
	cmpi	#DESK,msgbuf+6
	bne	m1
	cmpi	#ABOUT,msgbuf+8
	bne	end_menu
	
	move	#FORM_ALERT,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#1,int_in
	move.l	#alert3,addr_in
	jsr	aes
	bra	end_menu	
m1:
	cmpi	#FILE,msgbuf+6
	bne	end_menu
	
	cmpi	#QUIT,msgbuf+8
	bne	m2
	bra	done
m2:
	cmpi	#OPEN,msgbuf+8
	bne	m3
	bra	do_open
m3:
	cmpi	#CLOSE,msgbuf+8
	bne	m4
	jsr	do_close
	bra	end_menu	
m4:
	cmpi	#MDIALOG,msgbuf+8
	beq	do_dialog
	
end_menu:

; Turn off highlighted menu title.

	move	#MENU_TNORMAL,control
	move	#2,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	msgbuf+6,int_in
	move	#TRUE,int_in+2
	move.l	menu_addr,addr_in
	jsr	aes
	bra	get_event

do_full:

; Find window's current size.

	move	#WIND_GET,control
	move	#2,control+2
	move	#5,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_CURRXYWH,int_in+2
	jsr	aes
	move	int_out+2,d3		; Save window's current size.
	move	int_out+4,d4
	move	int_out+6,d5
	move	int_out+8,d6

; Check whether window already full size.

	cmp	fullx,d3
	bne	not_fulled
	cmp	fully,d4
	bne	not_fulled
	cmp	fullw,d5
	bne	not_fulled
	cmp	fullh,d6
	bne	not_fulled

; Window already full size, so set to previous size.
; First, get previous size of window.

	move	#WIND_GET,control
	move	#2,control+2
	move	#5,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_PREVXYWH,int_in+2
	jsr	aes
	move	int_out+2,prevx		; Save previous window size.
	move	int_out+4,prevy
	move	int_out+6,prevw
	move	int_out+8,prevh

; Show animated shrinking box.

	move	#GRAF_SHRINKBOX,control
	move	#8,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	prevx,int_in
	move	prevy,int_in+2
	move	prevw,int_in+4
	move	prevh,int_in+6
	move	fullx,int_in+8
	move	fully,int_in+10
	move	fullw,int_in+12
	move	fullh,int_in+14
	jsr	aes

; Set window to previous size.

	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_CURRXYWH,int_in+2
	move	prevx,int_in+4
	move	prevy,int_in+6
	move	prevw,int_in+8
	move	prevh,int_in+10
	jsr	aes
	bra	get_event

; Window not fulled, so set to full size.

not_fulled:

; Show animated expanding box.

	move	#GRAF_GROWBOX,control
	move	#8,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	d3,int_in
	move	d4,int_in+2
	move	d5,int_in+4
	move	d6,int_in+6
	move	fullx,int_in+8
	move	fully,int_in+10
	move	fullw,int_in+12
	move	fullh,int_in+14
	jsr	aes

; Set window to full size.

	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_CURRXYWH,int_in+2
	move	fullx,int_in+4
	move	fully,int_in+6
	move	fullw,int_in+8
	move	fullh,int_in+10
	jsr	aes
	bra	get_event
	
do_move:

; Check new window size against minimum allowable.

	cmp	#MIN_WIDTH,msgbuf+12
	bpl	width_ok
	move	#MIN_WIDTH,msgbuf+12
width_ok:
	cmp	#MIN_HEIGHT,msgbuf+14
	bpl	height_ok
	move	#MIN_HEIGHT,msgbuf+14

; Set window to new size or location.

height_ok:
	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_CURRXYWH,int_in+2
	move	msgbuf+8,int_in+4	; New window coords are in msgbuf.
	move	msgbuf+10,int_in+6
	move	msgbuf+12,int_in+8
	move	msgbuf+14,int_in+10
	jsr	aes
	bra	get_event

do_open:

; Get default directory path.

	move	#0,-(sp)
	move.l	#path,-(sp)
	move	#DGETPATH,-(sp)
	trap	#1
	addq.l	#8,sp

; Add filename mask to path.

	lea	path,a5
find_null:
	cmpi.b	#0,(a5)+
	bne	find_null
	subq.l	#1,a5
	move.b	#'\',(a5)+
	move.b	#'*',(a5)+
	move.b	#'.',(a5)+
	move.b	#'*',(a5)+
	move.b	#0,(a5)

; Call file selector box.

	move	#FSEL_INPUT,control
	move	#0,control+2
	move	#2,control+4
	move	#2,control+6
	move	#0,control+8
	move.l	#path,addr_in
	move.l	#filename,addr_in+4
	jsr aes

; Check whether user clicked OK button.

	cmpi	#1,int_out+2
	bne	end_menu

; Strip off default file filter from path string.

	lea	path,a5
find_null2:
	cmpi.b	#0,(a5)+
	bne	find_null2
find_backslash:
	cmpi.b	#'\',-(a5)
	bne	find_backslash
	addq.l	#1,a5

; Add file name to path.

	lea	filename,a6
add_filename:
	cmpi.b	#0,(a6)
	beq	file_added
	move.b	(a6)+,(a5)+
	bra	add_filename
file_added:

; Set window name to path name.

	move	#WIND_SET,control
	move	#6,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in		; Window's handle.
	move	#WF_NAME,int_in+2
	move.l	#path,int_in+4
	jsr	aes

; Open the new window.

	move	#WIND_OPEN,control
	move	#5,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in		; Window's handle.
	move	fullx,int_in+2
	move	fully,int_in+4
	move	fullw,int_in+6
	move	fullh,int_in+8
	jsr	aes
	move	#FALSE,closed

; Turn off "OPEN" menu item.

	move	#MENU_IENABLE,control
	move	#2,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#OPEN,int_in
	move	#FALSE,int_in+2
	move.l	menu_addr,addr_in
	jsr	aes

; Turn on "CLOSE" menu item.
	
	move	#CLOSE,int_in
	move	#TRUE,int_in+2
	jsr	aes
	bra	end_menu

do_dialog:

; Initialize the dialog's edit field.

	tedinfo_str dialog_addr,NAME
	move.b	#0,(a5)

; Initialize the dialog's buttons.

	move.l	dialog_addr,a3		; Get address of dialog.
	add.l	#(OK*24)+8,a3		; Calc address of OK ob_flags.
	move	#67,(a3)		; Set appropriate bits.
	add.l	#2,a3			; Calc address of ob_state.	
	move	#32,(a3)		; Set button to shadowed.

	move.l	dialog_addr,a3		; Get address of dialog.
	add.l	#(CANCEL*24)+8,a3	; Calc address of CANCEL ob_flags.
	move	#65,(a3)		; Set appropriate bits.
	add.l	#2,a3			; Calc address of ob_state.	
	move	#32,(a3)		; Set button to shadowed.

; Center dialog for screen.

        move	#FORM_CENTER,control
        move	#0,control+2
        move	#5,control+4
        move	#1,control+6
        move	#0,control+8
        move.l	dialog_addr,addr_in
        bsr	aes     
        move	int_out+2,dialx	; Save dialog's coordinates.
        move	int_out+4,dialy
        move	int_out+6,dialw
        move	int_out+8,dialh

; Reserve screen space for dialog box.

        move	#FORM_DIAL,control
        move	#9,control+2
        move	#1,control+4
        move	#0,control+6
        move	#0,control+8
        move	#FMD_START,int_in
        move	#0,int_in+2
        move	#0,int_in+4
        move	#10,int_in+6
        move	#10,int_in+8
        move	dialx,int_in+10
        move	dialy,int_in+12
        move	dialw,int_in+14
        move	dialh,int_in+16
        bsr	aes

; Animate expanding rectangle.

        move	#FMD_GROW,int_in	; Operation type.
        bsr	aes

; Draw dialog box on screen.
        
        move	#OBJC_DRAW,control
        move	#6,control+2
        move	#1,control+4
        move	#1,control+6
        move	#0,control+8
        move	#0,int_in
        move	#2,int_in+2
        move	dialx,int_in+4
        move	dialy,int_in+6
        move	dialw,int_in+8
        move	dialh,int_in+10
        move.l	dialog_addr,addr_in
        bsr	aes

; Activate dialog to accept user input.
      
        move	#FORM_DO,control
        move	#1,control+2
        move	#1,control+4
        move	#1,control+6
        move	#0,control+8
        move	#NAME,int_in
        move.l	dialog_addr,addr_in
        bsr	aes

; Determine which exit button clicked.

        cmpi	#OK,int_out
        bne	chk_but1

; OK button pressed.

	move	#FORM_ALERT,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#1,int_in
	move.l	#alert5,addr_in
	jsr	aes
	bra	dialog_done

; Cancel button pressed.

chk_but1:
	move	#FORM_ALERT,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#1,int_in
	move.l	#alert6,addr_in
	jsr	aes

dialog_done:

; Perform animated shrinking box.

        move	#FORM_DIAL,control
        move	#9,control+2
        move	#1,control+4
        move	#0,control+6
        move	#0,control+8
        move	#FMD_SHRINK,int_in
        move	#0,int_in+2
        move	#0,int_in+4
        move	#10,int_in+6
        move	#10,int_in+8
        move	dialx,int_in+10
        move	dialy,int_in+12
        move	dialw,int_in+14
        move	dialh,int_in+16
        bsr	aes

; Release dialog screen area.
        
        move	#FMD_FINISH,int_in
        bsr	aes
	bra	end_menu

do_redraw:

; Lock window from further drawing.

	move	#WIND_UPDATE,control
	move	#1,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	#BEG_UPDATE,int_in
	jsr	aes

; Shut off the mouse.

	move	#GRAF_MOUSE,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#M_OFF,int_in
	move.l	#0,addr_in
	jsr	aes

; Set writing mode to replace.

	move	#VSWR_MODE,control
	move	#0,control+2
	move	#1,control+6
	move	vws_handle,control+12
	move	#REPLACE,int_in
	jsr	vdi

; Get the size of the window's work area.

	move	#WIND_GET,control
	move	#2,control+2
	move	#5,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	move	#WF_WORKXYWH,int_in+2
	jsr	aes
	move	int_out+2,workx		; Save size of work area.
	move	int_out+4,worky
	move	int_out+6,workw
	move	int_out+8,workh

; Set fill pattern.

	move	#VSF_INTERIOR,control
	move	#0,control+2
	move	#1,control+6
	move	vws_handle,control+12
	move	#SOLID,int_in
	jsr	vdi
	
; Set fill style.

	move	#VSF_STYLE,control
	move	#0,control+2
	move	#1,control+6
	move	vws_handle,control+12
	move	#8,int_in
	jsr	vdi

; Set fill color.

	move	#VSF_COLOR,control
	move	#0,control+2
	move	#1,control+6
	move	vws_handle,control+12
	move	#WHITE,int_in
	jsr	vdi
	 
; Draw filled rectangle.

	move	#VR_RECFL,control
	move	#2,control+2
	move	#1,control+6
	move	vws_handle,control+12
	move	workx,pts_in
	move	worky,pts_in+2
	move	workx,d5		; The width and height...
	add	workw,d5		; must be translated from...
	subi	#1,d5			; AES type coordinates to...
	move	d5,pts_in+4		; VDI type coordinates.
	move	worky,d5
	add	workh,d5
	subi	#1,d5
	move	d5,pts_in+6
	jsr	vdi

; Unlock window.

	move	#WIND_UPDATE,control
	move	#1,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	#END_UPDATE,int_in
	jsr	aes
	
; Turn on the mouse.

	move	#GRAF_MOUSE,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#M_ON,int_in
	move.l	#0,addr_in
	jsr	aes
	bra	get_event

done:
		
; Erase the menu bar.

	move	#MENU_BAR,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#FALSE,int_in
	move.l	menu_addr,addr_in
	jsr	aes

; Close the window if it's still open.

	tst	closed
	bne	win_closed
	move	#WIND_CLOSE,control
	move	#1,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	jsr	aes

; Delete window from memory.
	
win_closed:
	move	#WIND_DELETE,control
	move	#1,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	jsr	aes

; Delete resource from memory.

	move	#RSRC_FREE,control
	move	#0,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	jsr	aes

; Close the virtual workstation.

close_down:
        move    #V_CLSVWK,control
        clr     control+2
        clr     control+6
        move    vws_handle,control+12
        jsr     vdi
                
; Close down application.

exit:
        move    #APPL_EXIT,control
        move    #0,control+2
        move    #1,control+4
        move    #0,control+6
        move    #0,control+8
        bsr     aes
        
end:
        move.w  #PTERM0,-(sp)           ; Back to desktop.
        trap    #1

;--------------------------------------------------------------------
; This subroutine closes the active window and sets the OPEN and
; CLOSE menu items appropriately.
;
; Input:        NONE
; Output:       NONE
; Regs changed: NONE
; Uses: apb, global, control, int_in, int_out, addr_in, closed, w_h1,
;        alert4, menu_addr
; Calls: aes
;--------------------------------------------------------------------
do_close:
        movem.l a0-a7/d0-d7,-(sp)       ; Save registers.

; Ask user if he really wants to close window.

	move	#FORM_ALERT,control
	move	#1,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#1,int_in
	move.l	#alert4,addr_in
	jsr	aes

; Check user's answer.

	cmpi	#YES,int_out		; Did user select YES button?
	bne	no_close

; If user answered yes, then close window.
	
	move	#WIND_CLOSE,control
	move	#1,control+2
	move	#1,control+4
	move	#0,control+6
	move	#0,control+8
	move	w_h1,int_in
	jsr	aes
	move	#TRUE,closed

; Turn on the "OPEN" menu selection.

	move	#MENU_IENABLE,control
	move	#2,control+2
	move	#1,control+4
	move	#1,control+6
	move	#0,control+8
	move	#OPEN,int_in
	move	#TRUE,int_in+2
	move.l	menu_addr,addr_in
	jsr	aes

; Turn off the "CLOSE" menu item.
	
	move	#CLOSE,int_in
	move	#FALSE,int_in+2
	jsr	aes

no_close:
        movem.l (sp)+,a0-a7/d0-d7       ; Restore registers.
	rts	

;--------------------------------------------------------------------
; This subroutine calls the AES.  Before calling this subroutine, the
; program must have correctly initialized the AES control, int_in,
; and addr_in arrays.
;
; Input:        Appropriate values in the int_in, addr_in, and
;               control arrays.
; Output:       Appropriate values in the int_out, addr_out, and
;               global arrays.
; Regs changed: NONE
; Uses: apb, global, int_in, int_out, addr_in, addr_out
;--------------------------------------------------------------------
aes:
        movem.l a0-a7/d0-d7,-(sp)       ; Save registers.
        move.l  #apb,d1                 ; Load addr of apb.
        move.w  #AES_OPCODE,d0          ; Load AES opcode.
        trap    #2                      ; Call AES.
        movem.l (sp)+,a0-a7/d0-d7       ; Restore registers.
        rts

;--------------------------------------------------------------------
; This subroutine calls the VDI. Before calling this subroutine, the
; program must have correctly initialized the VDI control, intin,
; and ptsin arrays.
;
; Input:        Appropriate values in the intin, ptsin, and
;               control arrays.
; Output:       Appropriate values in the intout and ptsout arrays.
; Regs changed: NONE
; Uses: vpb, intin, intout, ptsin, ptsout
;--------------------------------------------------------------------
vdi:
        movem.l a0-a7/d0-d7,-(sp)       ; Save registers.
        move.l  #vpb,d1                 ; Load addr of vpb.
        move.w  #VDI_OPCODE,d0          ; Load VDI opcode.
        trap    #2                      ; Call VDI.
        movem.l (sp)+,a0-a7/d0-d7       ; Restore registers.
        rts

        data
        
        even

apb:            dc.l    control,global,int_in,int_out,addr_in,addr_out
vpb:            dc.l    control,int_in,pts_in,int_out,pts_out

rsrcfile:	dc.b	"AESDEMO.RSC",0
wname:		dc.b	"Window Name",0
winfo:		dc.b	"Window Info Line",0
	
alert1:		dc.b	"[0][This demo runs|only in high or medium |"
		dc.b	"resolution.][  OK  ]",0
alert2:		dc.b	"[1][AESDEMO.RSC missing! ][Okay]",0
alert3:		dc.b	"[0][       AES Demo|   by Clayton Walnum"
		dc.b	"   ][CONTINUE]",0
alert4:		dc.b	"[2][Do you want to close|this window?]"
		dc.b	"[  YES  |  NO  ]",0
alert5:		dc.b	"[0][You clicked OK ][CONTINUE]",0
alert6:		dc.b	"[0][You clicked CANCEL ][CONTINUE]",0 
		
        bss

        even

menu_addr:	ds.l	1
dialog_addr:	ds.l	1
gr_handle       ds.w    1
vws_handle      ds.w    1
fullx:		ds.w	1
fully:		ds.w	1
fullw:		ds.w	1
fullh:		ds.w	1
workx:		ds.w	1
worky:		ds.w	1
workw:		ds.w	1
workh:		ds.w	1
currx:		ds.w	1
curry:		ds.w	1
currw:		ds.w	1
currh:		ds.w	1
prevx:		ds.w	1
prevy:		ds.w	1
prevw:		ds.w	1
prevh:		ds.w	1
dialx:		ds.w	1
dialy:		ds.w	1
dialw:		ds.w	1
dialh:		ds.w	1
w_h1:		ds.w	1
closed:		ds.w	1

msgbuf:		ds.w	8
path:		ds.w	128
filename:	ds.w	13
        
global:
ap_version:     ds.w    1
ap_count:       ds.w    1
ap_id:          ds.w    1
ap_private:     ds.l    1
ap_ptree:       ds.l    1
ap_1resv:       ds.l    1
ap_2resv:       ds.l    1
ap_3resv:       ds.l    1
ap_4resv:       ds.l    1

control:        ds.w    12
int_in:         ds.w    128
int_out:        ds.w    128
addr_in:        ds.l    64
addr_out:       ds.l    64
pts_in:         ds.w    128
pts_out:        ds.w    128

                ds.l    255
stack:          ds.l    1

